package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableModel;

import org.jdom.Element;

import translator.TranslationRule;
import util.TranslationTable;


/** 
 * This is the application dialog for rule editing
 * @author Jason. Park
 * @author Ki-Hyun. Sung
 * @author Edmund. Zurita
 * */
public class ConfigureRuleDialog extends JFrame {

	private final ConfigureRuleDialog me = this;

	private TranslationRule rule;

	private JLabel authorLabel = new JLabel("Author : ");
	private JLabel commentLabel = new JLabel("Comment : ");
	private JTextField authorField = new JTextField();
	private JTextField commentField = new JTextField();
	private JButton saveButton = new JButton("Apply");
	private JButton cancelButton = new JButton("Cancel");
	private JButton helpButton = new JButton("Help");

	private JPanel ruleTablePane;
	private TranslationTable ruleTable;


	/**
	 * Testing whether the dialog is properly layed out as per design
	 * @param args
	 */
	public static void main(String[] args){
		ConfigureRuleDialog dialog = new ConfigureRuleDialog("rules/BrailleRules.xml");
		dialog.setDefaultCloseOperation(JDialog.EXIT_ON_CLOSE);
		dialog.setVisible(true);
	}

	public ConfigureRuleDialog(TranslationRule rule){
		super("Edit Translation Rule");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setSize(370, 450);
		/*setLocation(
				owner.getX()+owner.getWidth()/2-getWidth()/2, 
				owner.getY()+owner.getHeight()/2-getHeight()/2);*/
		this.rule = rule;
		
		initComponents();
	}
	
	public ConfigureRuleDialog(String filepath){
		//super(owner, "Edit Translation Rule", false);
		super("Edit Translation Rule");
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		setSize(370, 450);
		setLocation(0,0);
		try {
			rule = new TranslationRule(filepath);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		initComponents();
	}
	
	/**
	 * Initializes components of the editor dialog
	 *
	 */
	private void initComponents(){
		authorField.setText(rule.getAuthor());
		commentField.setText(rule.getComment());

		Font buttonFont = new Font("Dialog", Font.BOLD, 11);

		saveButton.setFont(buttonFont);
		saveButton.setMargin(new Insets(2,10,2,10));
		saveButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				Element root = rule.getDocument().getRootElement();
				root.setAttribute("Author", authorField.getText());
				root.setAttribute("Comment", commentField.getText());
				
				Element newRules = new Element("TranslationRules");
				String sound, action;
				Element tempElem;
				
				Hashtable<String,Element> soundTable = rule.getHashtable();
				soundTable.clear();
				for(int i = 0; i < ruleTable.getRowCount(); i++){
					sound = (String)ruleTable.getValueAt(i, 0);
					action = (String)ruleTable.getValueAt(i, 1);
					tempElem = new Element("Rule");
					tempElem.setAttribute("Sound", sound);
					tempElem.setAttribute("Action", action);
					newRules.addContent(tempElem);
					soundTable.put(sound, tempElem);
				}
				rule.getDocument().getRootElement().removeChildren("TranslationRules");
				rule.getDocument().getRootElement().addContent(newRules);
				rule.refreshTable();
				
				me.dispose();
				
			}
		});
		
		cancelButton.setFont(buttonFont);
		cancelButton.setMargin(new Insets(2,10,2,10));
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				me.dispose();
			}
		});
		
		helpButton.setFont(buttonFont);
		helpButton.setMargin(new Insets(2,10,2,10));

		initTablePane();
		allocateComponents();
	}
	
	/**
	 * Initializes components of the central JPane
	 */
	private void initTablePane() {
		ruleTable = new TranslationTable(rule);
		
		JScrollPane tableScrollPane = new JScrollPane(ruleTable);
		tableScrollPane.setSize(new Dimension(255, 83));
		tableScrollPane.setVerticalScrollBarPolicy(
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		
		JButton addRuleButton = new JButton("Add New Rule");
		JButton removeRuleButton = new JButton("Remove Selected Rules");
		
		Font smallFont = new Font("Dialog", Font.PLAIN, 10);
		addRuleButton.setFont(smallFont);
		removeRuleButton.setFont(smallFont);
		addRuleButton.setMargin(new Insets(2,5,2,5));
		removeRuleButton.setMargin(new Insets(2,5,2,5));
		addRuleButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DialogFactory.createAddRuleDialog(me, ruleTable);
			}
		});
		
		removeRuleButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int[] selRows = ruleTable.getSelectedRows();
				if(selRows.length > 0){
					ruleTable.clearSelection();
					for(int i = selRows.length-1; i > -1 ; i--){
						((DefaultTableModel)ruleTable.getModel()).removeRow(selRows[i]);
					}
					ruleTable.addNotify();
				}
			}
		});
		
		ruleTablePane = new JPanel(); 
		ruleTablePane.setSize(new Dimension(285, 150));
		
		/**
		 * tpLayout stands for TablePaneLayout
		 */
		ruleTablePane.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		ruleTablePane.setBorder(
				BorderFactory.createTitledBorder(
						null, 
						" Translation Rules ", 
						TitledBorder.DEFAULT_JUSTIFICATION, 
						TitledBorder.DEFAULT_POSITION, 
						smallFont, 
						new Color(51, 51, 51)));
		
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(5, 5, 5, 5);
		ruleTablePane.add(tableScrollPane, c);
		
		JPanel addRemoveRulePane = new JPanel(new GridLayout(1, 2, 70, 0));
		addRemoveRulePane.add(addRuleButton);
		addRemoveRulePane.add(removeRuleButton);
		
		c.gridy = 1;
		c.insets = new Insets(0, 70, 5, 70);
		ruleTablePane.add(addRemoveRulePane, c);

	}
	/**
	 * Gets rule table column headings
	 * @return Vector<String> Sound Input, Action
	 */
	private Vector<String> getColumnHeading(){
		Vector<String> columnHeading = new Vector<String>();
		columnHeading.add("Sound Input");
		columnHeading.add("Action");
		return columnHeading;
	}

	/**
	 * Get Vector containing Vectors defining each rule
	 * @return Vector<Vector> Each Vector contains String Sound (Key) and String Action
	 */
	private Vector<Vector> getRulesVector(){
		Vector<Vector> ruleVector = new Vector<Vector>();

		try{
			Vector<String> singleRule;

			for(Map.Entry<String, Element> m : rule.getHashtable().entrySet()){
				singleRule = new Vector<String>();
				singleRule.add(m.getKey());
				singleRule.add(m.getValue().getAttributeValue("Action"));
				ruleVector.add(singleRule);
			}
		}
		catch(NullPointerException e){}

		return ruleVector;
	}

	/**
	 * Adds components to editor layout
	 *
	 */
	private void allocateComponents() {
		JPanel container = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;
		
		c.weightx = 0;
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(20, 30, 5, 5);
		container.add(authorLabel, c);
		
		c.weightx = 4;
		c.gridx = 1;
		c.insets = new Insets(20, 0, 5, 30);
		container.add(authorField, c);
		
		c.weightx = 0;
		c.gridy = 1;
		c.gridx = 0;
		c.insets = new Insets(0, 30, 5, 5);
		container.add(commentLabel, c);
		
		c.weightx = 4;
		c.gridx = 1;
		c.insets = new Insets(0, 0, 5, 30);
		container.add(commentField, c);
		
		c.weightx = 0;
		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(0, 10, 5, 10);
		container.add(ruleTablePane, c);
		
		JPanel buttonPane = new JPanel(new GridLayout(1, 3, 30, 0));
		buttonPane.add(saveButton);
		buttonPane.add(cancelButton);
		buttonPane.add(helpButton);
		
		c.gridy = 3;
		c.insets = new Insets(0, 70, 10, 70);
		container.add(buttonPane, c);
		
		
		setContentPane(container);
		pack();
		setVisible(true);
	}
	
	public TranslationRule getTranslationRule(){
		return rule;
	}
}

/** 
 * This is the dialog for adding rules
 * @author Jason. Park
 * @author Ki-Hyun. Sung
 * @author Edmund. Zurita
 * */
class DialogFactory {
	
	private static JLabel soundLabel;
	private static JLabel actionLabel;
	private static JTextField soundField;
	private static JTextField actionField;
	private static JButton addButton;
	private static JButton cancelButton;
	
	private static JTable table;
	
	private static JDialog dialog;
	
	public DialogFactory(){
	}
	
	/**
	 * Creates Add Rule Dialog
	 * @param owner Main application JFrame
	 * @param t Rules table
	 * @return JDialog Add Rule Dialog
	 */
	public static JDialog createAddRuleDialog(JFrame owner, JTable t){
		dialog = new JDialog(owner, "Add New Translation Rule", true);
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.setSize(245, 135);
		dialog.setLocation(
				owner.getX()+owner.getWidth()/2-dialog.getWidth()/2, 
				owner.getY()+owner.getHeight()/2-dialog.getHeight()/2);
		
		table = t;
		
		soundLabel = new JLabel("Sound : ");
		actionLabel = new JLabel("Action : ");
		soundField = new JTextField();
		actionField = new JTextField();
		addButton = new JButton("Add");
		cancelButton = new JButton("Cancel");
		
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DefaultTableModel model = (DefaultTableModel)table.getModel();
				Vector<String> v = new Vector<String>();
				v.add(TranslationRule.sortSequence(soundField.getText()));
				v.add(actionField.getText());
				model.addRow(v);
				table.addNotify();
				dialog.dispose();
			}
		});
		
		cancelButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				dialog.dispose();
			}
		});
		
		dialog.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		
		c.weightx = 0;
		c.insets = new Insets(10, 10, 0, 10);
		dialog.add(soundLabel, c);
		
		c.weightx = 5;
		c.gridx = 1;
		c.insets = new Insets(10, 5, 10, 10);
		dialog.add(soundField, c);
		
		c.weightx = 0;
		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(0, 10, 10, 0);
		dialog.add(actionLabel, c);
		
		c.weightx = 5;
		c.gridx = 1;
		c.insets = new Insets(0, 5, 10, 10);
		dialog.add(actionField, c);
		
		JPanel addCancelButtonPane = new JPanel(new GridLayout(1, 2, 15, 0));
		addCancelButtonPane.add(addButton);
		addCancelButtonPane.add(cancelButton);
		
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 2;
		c.insets = new Insets(5, 30, 10, 30);
		dialog.add(addCancelButtonPane, c);
		
		dialog.setVisible(true);
		dialog.pack();
		
		
		return dialog;
	}
}
