package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Ellipse2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.TitledBorder;

import org.jdom.JDOMException;

import receiver.DumpReceiver;
import translator.TranslationRule;
import util.TranslationTable;
import data.TestRecord;

public class PracticeProgram extends JFrame{

	public PracticeProgram(String name){
		super("MIDI QWERTYFIER Performance Test Program");
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				if(record.recordTable.size() > 0)
					record.outputToFile();
				System.exit(0);
			}
		});
		
		loadSamples();
		
		userName = name;
		record = new TestRecord(userName+"_T_T");
		xmlFilePath = "rules/BrailleRules.xml";
		try {
			rule = new TranslationRule(xmlFilePath);
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (JDOMException e1) {
			e1.printStackTrace();
		}
		ruleTable = new TranslationTable(rule);

		//initMenu();
		initComponents();
	}

	private void loadSamples() {
		File file = new File("sample/SENTENCE.TXT");
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

		BufferedReader reader = new BufferedReader(new InputStreamReader(fis));

		try {
			while(reader.ready()){
				sampleLines.add(reader.readLine());
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void initMenu(){
		JMenuBar menu = new JMenuBar();
		
		JMenu file = new JMenu("File");
		JMenuItem createAccount = new JMenuItem("Create account");
		createAccount.setActionCommand("Create");
		JMenuItem loadAccount = new JMenuItem("Load account");
		loadAccount.setActionCommand("Load");
		JMenuItem saveAccount = new JMenuItem("Save session");
		saveAccount.setActionCommand("Save");
		JMenuItem exit = new JMenuItem("Exit");
		exit.setActionCommand("Exit");
		JMenu ruleMenu = new JMenu("Rule");
		JMenuItem loadRule = new JMenuItem("Load rule");
		
		file.add(createAccount);
		file.add(loadAccount);
		file.add(saveAccount);
		file.add(exit);

		ruleMenu.add(loadRule);

		menu.add(file);

		setJMenuBar(menu);
	}

	private void initComponents(){
		initInformationPane();
		initStatePane();
		initTextPane();
		initBarPane();
		initBraillePane();
		initRulePane();

		JPanel topPane = new JPanel(new BorderLayout());
		topPane.add(informationPane, BorderLayout.WEST);
		topPane.add(statePane, BorderLayout.EAST);

		JPanel mainPane = new JPanel(new BorderLayout());

		mainPane.add(topPane, BorderLayout.NORTH);
		mainPane.add(textPane, BorderLayout.CENTER);

		JPanel southPane = new JPanel(new BorderLayout());
		southPane.add(barPane, BorderLayout.CENTER);
		southPane.add(braillePane, BorderLayout.EAST);

		mainPane.add(southPane, BorderLayout.SOUTH);

		contentPane.add(mainPane, BorderLayout.CENTER);
		contentPane.add(rulePane, BorderLayout.EAST);

		setContentPane(contentPane);
		setSize(1100, 500);
		setVisible(true);
		
		char c = sampleText.getText().charAt(inputLabel.getText().length());
		braillePane.updateIndicator(""+c);
	}

	private void initInformationPane() {
		initDeviceBox();

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 20;
		c.ipady = 10;
		c.insets = new Insets(0, 10, 0, 0);
		JLabel tempLabel = new JLabel("User Name");
		informationPane.add(tempLabel, c);

		c.gridx = 1;
		userLabel.setText(userName);
		informationPane.add(userLabel, c);

		tempLabel = new JLabel("Translation Rule");
		c.gridx = 0;
		c.gridy = 1;
		informationPane.add(tempLabel, c);

		c.gridx = 1;
		informationPane.add(ruleLabel, c);
		ruleLabel.setText(xmlFilePath);

		tempLabel = new JLabel("Device List");
		c.gridx = 0;
		c.gridy = 2;
		informationPane.add(tempLabel, c);

		c.gridx = 1;
		informationPane.add(devices, c);
	}

	private void initStatePane() {
		brailleBox = new JCheckBox("Braille Keys", true);
		ruleBox = new JCheckBox("Display Rules", true);

		class CheckBoxListener implements ItemListener {
			public void itemStateChanged(ItemEvent e) {
				if (e.getSource() == brailleBox){
					isBrailleBoxTicked = (e.getStateChange() == ItemEvent.DESELECTED ? false : true);
					braillePane.repaint();
				}
				else if (e.getSource() == ruleBox) {
					if(e.getStateChange() == ItemEvent.DESELECTED){
						contentPane.remove(rulePane);
						
						Dimension s = getSize();
						pack();
						setSize(s.width-300, s.height);
					}
					else if(e.getStateChange() == ItemEvent.SELECTED) {
						contentPane.add(rulePane, BorderLayout.EAST);
						
						Dimension s = getSize();
						pack();
						setSize(s.width+300, s.height);
					}
				}
				
				if(record.recordTable.size() > 0){
					record.outputToFile();
				}
				
				sampleText.setText(getRandomLine());
				
				record = new TestRecord(userName+"_"
						+(ruleBox.isSelected() ? "T" : "F") +"_"
						+(brailleBox.isSelected() ? "T" : "F"));
			}
		}

		CheckBoxListener boxListener = new CheckBoxListener();

		brailleBox.addItemListener(boxListener);
		ruleBox.addItemListener(boxListener);

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 50;

		statePane.add(brailleBox, c);

		c.gridy = 1;
		statePane.add(ruleBox, c);
	}

	private void initDeviceBox() {
		Vector<String> deviceList = new Vector<String>();
		try {
			String[] argument = {"-l"};
			deviceList = receiver.MidiInDump.showDevices(argument);
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		devices = new JComboBox(deviceList);
		devices.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(devices.getSelectedIndex() == 0){
					if(simulator != null){
						simulator.close();
						simulator = null;
					}
					if(r != null){
						r.close();
						r = null;

					}
				}
				else if(devices.getSelectedIndex() > 1){
					if(simulator != null){
						simulator.close();
						simulator = null;
					}
					r =
						(DumpReceiver)(receiver.MidiInDump.selectDevice(Integer.parseInt(
								((String)devices.getSelectedItem()).substring(0,1))));
					r.setPracticeProgram(me);
					//receiver.MidiInDump.run(inputDevice,r);
				}
				else if(devices.getSelectedIndex() == 1){
					if(r != null){
						r.close();
						r = null;
					}
					simulator = new SoundSimulator(me);
				}
			}});
	}

	private void initTextPane() {
		textPane.setBorder(BorderFactory.createTitledBorder(
				null, 
				" Input Field ", 
				TitledBorder.DEFAULT_JUSTIFICATION, 
				TitledBorder.DEFAULT_POSITION, 
				new Font("Dialog", Font.PLAIN, 10), 
				new Color(51, 51, 51)));

		Font largeFont = new Font("Dialog", Font.PLAIN, 20);
		Font largeBoldFont = new Font("Dialog", Font.BOLD, 23);
		hiddenText_1.setFont(largeFont);
		hiddenText_2.setFont(largeFont);
		sampleText.setFont(largeBoldFont);
		inputLabel.setFont(largeBoldFont);

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.WEST;
		c.gridy = 0;
		textPane.add(hiddenText_1, c);

		c.gridy = 1;
		c.ipady = 5;
		sampleText.setText(getRandomLine());
		
		textPane.add(sampleText, c);

		c.gridy = 2;
		c.ipady = 0;
		textPane.add(inputLabel, c);

		c.gridy = 3;
		textPane.add(hiddenText_2, c);
	}

	private void initBarPane() {
		barPane.setBorder(BorderFactory.createTitledBorder(
				null, 
				" Performance indicators ", 
				TitledBorder.DEFAULT_JUSTIFICATION, 
				TitledBorder.DEFAULT_POSITION, 
				new Font("Dialog", Font.PLAIN, 10), 
				new Color(51, 51, 51)));

		JLabel label;
		bestRecord = new JProgressBar(0, 100);
		previousRecord = new JProgressBar(0 , 100);
		accuracyBar = new JProgressBar(0, 100);

		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		label = new JLabel("Today's Best: ");
		barPane.add(label, c);

		c.gridx = 1;
		barPane.add(bestRecord, c);

		c.gridx = 0;
		c.gridy = 1;
		label = new JLabel("Previous record: ");
		barPane.add(label, c);

		c.gridx = 1;
		barPane.add(previousRecord, c);

		c.gridx = 0;
		c.gridy = 2;
		label = new JLabel("Accuracy: ");
		barPane.add(label, c);

		c.gridx = 1;
		barPane.add(accuracyBar, c);
	}

	private void initBraillePane() {
		braillePane = new BrailleDisplay();
	}

	private void initRulePane() {
		ruleTable.setEnabled(false);
		rulePane = new JScrollPane(ruleTable);
		rulePane.setPreferredSize(new Dimension(300,100));
		rulePane.setVerticalScrollBarPolicy(
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		rulePane.setBorder(
				BorderFactory.createTitledBorder(
						null, 
						" Translation Rules ", 
						TitledBorder.DEFAULT_JUSTIFICATION, 
						TitledBorder.DEFAULT_POSITION, 
						new Font("Dialog", Font.PLAIN, 10), 
						new Color(51, 51, 51)));
	}

	private String getRandomLine(){
		int randomIndex = (int) (Math.random() * sampleLines.size());

		return sampleLines.get(randomIndex);  
	}

	private void updatePerformanceBar(long currentTime){
		long time = currentTime - startTime;
		
		String sample = sampleText.getText();
		String input = inputLabel.getText();
		
		int performance = (int) time / sample.length();
		
		if(performance > todayBest){
			todayBest = performance;
			DefaultBoundedRangeModel rangeModel = new DefaultBoundedRangeModel(0, 0, 0, todayBest);
			previousRecord.setModel(rangeModel);
			bestRecord.setModel(rangeModel);
			bestRecord.setValue(todayBest);
		}
		previousRecord.setValue(performance);
		
		int correctCharCtr = 0;
		for(int i = 0; i < sample.length(); i++){
			//System.out.println(sample.substring(i, i+1) + "_"+input.substring(i, i+1));
			if(sample.substring(i, i+1).equalsIgnoreCase(input.substring(i, i+1))){
				correctCharCtr++;
			}
		}
		
		double result = (double)correctCharCtr/sample.length();
		int accuracy = (int)(result * 100);
		System.out.println(accuracy);
		accuracyBar.setValue((int)accuracy);
		
		startTime = currentTime;
	}

	public static void main(String[] args){
		String userName = "PracticeProgram";
		if(args.length == 1)
			userName = args[0];
		
		new PracticeProgram(userName);
	}

	public void update(String inputSound){
		String action = rule.getAction(inputSound);

		long currentTime = System.currentTimeMillis();
		if(previousTime != -1 && !action.equals(""))
			record.addData(action, currentTime - previousTime);
		previousTime = currentTime;

		if(inputLabel.getText().length() == 0)
			startTime = currentTime;

		inputLabel.setText(inputLabel.getText()+action);

		if(sampleText.getText().length() == inputLabel.getText().length()){
			updatePerformanceBar(currentTime);
			
			sampleText.setText(getRandomLine());
			inputLabel.setText("");
			previousTime = -1;
		}
		char c = sampleText.getText().charAt(inputLabel.getText().length());
		braillePane.updateIndicator(""+c);
	}

	private String xmlFilePath;
	private String userName = "";
	
	private JLabel ruleLabel = new JLabel("");
	private JLabel userLabel = new JLabel("");
	private JComboBox devices;
	private JCheckBox brailleBox, ruleBox;

	private JLabel hiddenText_1 = new JLabel(" ");
	private JLabel hiddenText_2 = new JLabel(" ");
	private JLabel sampleText = new JLabel("Sample Text");;
	private JLabel inputLabel = new JLabel("");

	private JProgressBar bestRecord;
	private JProgressBar previousRecord;
	private JProgressBar accuracyBar;

	private JPanel contentPane = new JPanel(new BorderLayout());
	private JPanel informationPane = new JPanel(new GridBagLayout());
	private JPanel statePane = new JPanel(new GridBagLayout());
	private JPanel textPane = new JPanel(new GridBagLayout());
	private JPanel barPane = new JPanel(new GridBagLayout());
	private JScrollPane rulePane;
	private BrailleDisplay braillePane;

	private DumpReceiver r =  new DumpReceiver();
	private SoundSimulator simulator;
	private TranslationTable ruleTable;

	private TestRecord record;
	private long previousTime = -1;
	private long startTime = 0;
	private int todayBest = 0;

	private boolean isBrailleBoxTicked = true;

	private TranslationRule rule;
	private Vector<String> sampleLines = new Vector<String>();

	private PracticeProgram me = this;

	private class BrailleDisplay extends JPanel{
		Shape shapes[] = new Shape[7];
		boolean pressedKeys[] = new boolean[7];

		Color green = Color.green;
		Color red = Color.red;
		Color gray = Color.gray;

		public BrailleDisplay(){
			setPreferredSize(new Dimension(80, 80));

			setBorder(BorderFactory.createTitledBorder(
					null, 
					" Braille Keys ", 
					TitledBorder.DEFAULT_JUSTIFICATION, 
					TitledBorder.DEFAULT_POSITION, 
					new Font("Dialog", Font.PLAIN, 10), 
					new Color(51, 51, 51)));

			shapes[0] = new Ellipse2D.Double(15.0, 20.0, 10.0, 10.0);
			shapes[1] = new Ellipse2D.Double(15.0, 35.0, 10.0, 10.0);
			shapes[2] = new Ellipse2D.Double(15.0, 50.0, 10.0, 10.0);

			shapes[3] = new Ellipse2D.Double(55.0, 20.0, 10.0, 10.0);
			shapes[4] = new Ellipse2D.Double(55.0, 35.0, 10.0, 10.0);
			shapes[5] = new Ellipse2D.Double(55.0, 50.0, 10.0, 10.0);

			shapes[6] = new Rectangle(20, 67, 42, 6);
		}

		public void updateIndicator(String word){
			String sound = rule.getSound(word);
			if(sound == null){
				sound = rule.getSound(word.toLowerCase());
			}
			if(sound == null)
				return;
			
			for(int i = 0; i < pressedKeys.length; i++){
				pressedKeys[i] = false;
			}
			
			//Default
			if(sound.contains("A4"))
				pressedKeys[0] = true;
			if(sound.contains("F4"))
				pressedKeys[1] = true;
			if(sound.contains("D4"))
				pressedKeys[2] = true;
			if(sound.contains("E5"))
				pressedKeys[3] = true;
			if(sound.contains("G5"))
				pressedKeys[4] = true;
			if(sound.contains("B5"))
				pressedKeys[5] = true;
			if(sound.contains("C5"))
				pressedKeys[6] = true;
			
			//Beryl
//			if(sound.contains("B3"))
//				pressedKeys[0] = true;
//			if(sound.contains("A3"))
//				pressedKeys[1] = true;
//			if(sound.contains("G3"))
//				pressedKeys[2] = true;
//			if(sound.contains("E4"))
//				pressedKeys[3] = true;
//			if(sound.contains("F4"))
//				pressedKeys[4] = true;
//			if(sound.contains("G4"))
//				pressedKeys[5] = true;
//			if(sound.contains("C4"))
//				pressedKeys[6] = true;
			
			//Gerald
//			if(sound.contains("B4"))
//				pressedKeys[0] = true;
//			if(sound.contains("A4"))
//				pressedKeys[1] = true;
//			if(sound.contains("F4"))
//				pressedKeys[2] = true;
//			if(sound.contains("D5"))
//				pressedKeys[3] = true;
//			if(sound.contains("E5"))
//				pressedKeys[4] = true;
//			if(sound.contains("F5"))
//				pressedKeys[5] = true;
//			if(sound.contains("C5"))
//				pressedKeys[6] = true;
			
			


			repaint();
		}

		public void paintComponent(Graphics g){
			super.paintComponent(g);

			Graphics2D g2 = (Graphics2D)g;

			if(!isBrailleBoxTicked){
				g2.setColor(gray);
				for(int i = 0; i < shapes.length; i++){
					g2.fill(shapes[i]);
				}
			}
			else{
				for(int i = 0; i < shapes.length; i++){
					if(pressedKeys[i])
						g2.setColor(green);

					else
						g2.setColor(red);

					g2.fill(shapes[i]);
				}
			}
		}
	}
}