package receiver;

/*
 *	DumpReceiver.java
 *
 *	This file is part of jsresources.org
 */

/*
 * Copyright (c) 1999 - 2001 by Matthias Pfisterer
 * Copyright (c) 2003 by Florian Bomers
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import gui.MIDIQwertyfier;
import gui.PracticeProgram;

import	java.io.PrintStream;
import java.util.ArrayList;
import	javax.sound.midi.MidiMessage;
import	javax.sound.midi.ShortMessage;
import	javax.sound.midi.Receiver;

import translator.TranslationRule;

/** 
 * This is a utility class for obtaining MIDI devices connected to the computer.
 * @author Jason. Park
 * @author Ki-Hyun. Sung
 * @author Edmund. Zurita
 * */
public class DumpReceiver
implements	Receiver
{

	/*	public static long seByteCount = 0;
	public static long smByteCount = 0;
	public static long seCount = 0;
	public static long smCount = 0;*/



	//private static final String[]		sm_astrKeySignatures = {"Cb", "Gb", "Db", "Ab", "Eb", "Bb", "F", "C", "G", "D", "A", "E", "B", "F#", "C#"};
	/*	private static final String[]		SYSTEM_MESSAGE_TEXT =
	{
		"System Exclusive (should not be in ShortMessage!)",
		"MTC Quarter Frame: ",
		"Song Position: ",
		"Song Select: ",
		"Undefined",
		"Undefined",
		"Tune Request",
		"End of SysEx (should not be in ShortMessage!)",
		"Timing clock",
		"Undefined",
		"Start",
		"Continue",
		"Stop",
		"Undefined",
		"Active Sensing",
		"System Reset"
	};*/

	/*	public long getSeCount(){
		return seCount;
	}

	public long getSmCount(){
		return smCount;
	}*/

	/*	private static final String[]		QUARTER_FRAME_MESSAGE_TEXT =
	{
		"frame count LS: ",
		"frame count MS: ",
		"seconds count LS: ",
		"seconds count MS: ",
		"minutes count LS: ",
		"minutes count MS: ",
		"hours count LS: ",
		"hours count MS: "
	};

	private static final String[]		FRAME_TYPE_TEXT =
	{
		"24 frames/second",
		"25 frames/second",
		"30 frames/second (drop)",
		"30 frames/second (non-drop)",
	};*/

	private PrintStream		m_printStream;
	public boolean			m_bDebug;
	public boolean 			m_testing;
	private boolean			m_bPrintTimeStampAsTicks;
	private MIDIQwertyfier	midiApp;
	private PracticeProgram practiceApp;
	public ArrayList<MidiMessage> m= new ArrayList<MidiMessage>();
	public ArrayList<Long> n= new ArrayList<Long>(); 

	public TranslationRule rule; 
	//allowance is 1 second (given in microseconds) 
//	public long allowance = 1000000;


	/**
	 * Constructs a DumpReceiver with the specified printstream.
	 */
	public DumpReceiver(PrintStream printStream, boolean testing)
	{
		this(printStream, false, testing);


	}

	/**
	 * Constructs a DumpReceiver with the specified printstream and the option to print 
	 * timestamps to the printstream.
	 */
	public DumpReceiver(PrintStream printStream,
			boolean bPrintTimeStampAsTicks, boolean testing)
	{


		m_testing=testing;
		m_printStream = printStream;
		m_bDebug = false;
		m_bPrintTimeStampAsTicks = bPrintTimeStampAsTicks;
	}

	/**
	 * Constructs a DumpReceiver.
	 */
	public DumpReceiver(){

	}

	/*	public String recognizeChord(ArrayList m,ArrayList n){
		long allowance = 1000;
		for(int i=0; i<n.size(); i++){
			Long l = (Long)(n.get(i));
			System.out.println(l.longValue());


			//System.out.println(d.longValue()); 

		}
		//System.out.println(m.size());
		//System.out.println(n.size());
		return "{CEG}";		
	}*/




	public void close()
	{
		this.midiApp = null;
	}

	/**
	 * Sets the midiApp variable to the specified application instant.
	 * @param midiApp - The Application instant
	 */
	public void setMIDIApp(MIDIQwertyfier midiApp){
		this.midiApp = midiApp;
	}

	public void setPracticeProgram(PracticeProgram practiceApp){
		this.practiceApp = practiceApp;
	}
	
	/**
	 * Automatically invoked when a MidiEvent is received. Builds up a sequence of MidiEvents and 
	 * its corresponding timestamps in two arrays until the sequence ends. 
	 * Also dumps the events onto the console. 
	 * @param message - The ShortMessage received
	 * @param lTimeStamp - The time in which the ShortMessage was received.
	 */
	public void send(MidiMessage message, long lTimeStamp){


		//System.out.println(m_bDebug);
		m.add(message);
		n.add(lTimeStamp);
		ShortMessage msg = (ShortMessage)message;
		if(msg.getCommand()==128 && m.size() > 1){
			//System.out.println(m_bDebug);

			//if(m_testing=false){
			String s = MidiInDump.getSequence(this);
			String seq = TranslationRule.sortSequence(s);
			if(midiApp != null){
				System.out.println("[sound] 1 "+seq);
				midiApp.update(seq);
			}
			else if(practiceApp != null){
				System.out.println("[sound] 2 "+seq);
				practiceApp.update(seq);
			}
			else{
				System.out.println("[sound] 3 "+seq);
				String sound = seq;
				
				
				//System.out.println(sound);
				//System.out.println(rule.getAction(sound));			
			}
		}

		if(m.size()==1 && ((ShortMessage)m.get(0)).getCommand()==128)
			m.clear();



		m_bPrintTimeStampAsTicks = true;
		String	strMessage = null;

		//System.out.println(message instanceof ShortMessage);

		if (message instanceof ShortMessage)
		{
			strMessage = MidiCommon.decodeMessage((ShortMessage) message);
		}
		/*		else if (message instanceof SysexMessage)
		{
			strMessage = decodeMessage((SysexMessage) message);
		}
		else if (message instanceof MetaMessage)
		{
			strMessage = decodeMessage((MetaMessage) message);
		}*/
		else
		{
			strMessage = "unknown message type";
		}
		String	strTimeStamp = null;

		//System.out.println(m_bPrintTimeStampAsTicks);
		if (m_bPrintTimeStampAsTicks)
		{
			strTimeStamp = "tick " + lTimeStamp + ": ";
		}
		else
		{
			if (lTimeStamp == -1L)
			{
				strTimeStamp = "timestamp [unknown]: ";
			}
			else
			{
				strTimeStamp = "timestamp " + lTimeStamp + " us: ";
			}
		}

		//System.out.println(strTimeStamp);
		//System.out.println(strMessage);
		if(m_bDebug=false){
			m_printStream.println(strTimeStamp + strMessage);
		}
	}






	/*	public String decodeMessage(SysexMessage message)
	{
		byte[]	abData = message.getData();
		String	strMessage = null;
		// System.out.println("sysex status: " + message.getStatus());
		if (message.getStatus() == SysexMessage.SYSTEM_EXCLUSIVE)
		{
			strMessage = "Sysex message: F0" + getHexString(abData);
		}
		else if (message.getStatus() == SysexMessage.SPECIAL_SYSTEM_EXCLUSIVE)
		{
			strMessage = "Continued Sysex message F7" + getHexString(abData);
			seByteCount--; // do not count the F7
		}
		seByteCount += abData.length + 1;
		seCount++; // for the status byte
		return strMessage;
	}



	public String decodeMessage(MetaMessage message)
	{
		byte[]	abMessage = message.getMessage();
		byte[]	abData = message.getData();
		int	nDataLength = message.getLength();
		String	strMessage = null;
		// System.out.println("data array length: " + abData.length);
		switch (message.getType())
		{
		case 0:
			int	nSequenceNumber = ((abData[0] & 0xFF) << 8) | (abData[1] & 0xFF);
			strMessage = "Sequence Number: " + nSequenceNumber;
			break;

		case 1:
			String	strText = new String(abData);
			strMessage = "Text Event: " + strText;
			break;

		case 2:
			String	strCopyrightText = new String(abData);
			strMessage = "Copyright Notice: " + strCopyrightText;
			break;

		case 3:
			String	strTrackName = new String(abData);
			strMessage = "Sequence/Track Name: " +  strTrackName;
			break;

		case 4:
			String	strInstrumentName = new String(abData);
			strMessage = "Instrument Name: " + strInstrumentName;
			break;

		case 5:
			String	strLyrics = new String(abData);
			strMessage = "Lyric: " + strLyrics;
			break;

		case 6:
			String	strMarkerText = new String(abData);
			strMessage = "Marker: " + strMarkerText;
			break;

		case 7:
			String	strCuePointText = new String(abData);
			strMessage = "Cue Point: " + strCuePointText;
			break;

		case 0x20:
			int	nChannelPrefix = abData[0] & 0xFF;
			strMessage = "MIDI Channel Prefix: " + nChannelPrefix;
			break;

		case 0x2F:
			strMessage = "End of Track";
			break;

		case 0x51:
			int	nTempo = ((abData[0] & 0xFF) << 16)
					| ((abData[1] & 0xFF) << 8)
					| (abData[2] & 0xFF);           // tempo in microseconds per beat
			float bpm = convertTempo(nTempo);
			// truncate it to 2 digits after dot
			bpm = (float) (Math.round(bpm*100.0f)/100.0f);
			strMessage = "Set Tempo: "+bpm+" bpm";
			break;

		case 0x54:
			// System.out.println("data array length: " + abData.length);
			strMessage = "SMTPE Offset: "
				+ (abData[0] & 0xFF) + ":"
				+ (abData[1] & 0xFF) + ":"
				+ (abData[2] & 0xFF) + "."
				+ (abData[3] & 0xFF) + "."
				+ (abData[4] & 0xFF);
			break;

		case 0x58:
			strMessage = "Time Signature: "
				+ (abData[0] & 0xFF) + "/" + (1 << (abData[1] & 0xFF))
				+ ", MIDI clocks per metronome tick: " + (abData[2] & 0xFF)
				+ ", 1/32 per 24 MIDI clocks: " + (abData[3] & 0xFF);
			break;

		case 0x59:
			String	strGender = (abData[1] == 1) ? "minor" : "major";
			strMessage = "Key Signature: " + sm_astrKeySignatures[abData[0] + 7] + " " + strGender;
			break;

		case 0x7F:
			// TODO: decode vendor code, dump data in rows
			String	strDataDump = getHexString(abData);
			strMessage = "Sequencer-Specific Meta event: " + strDataDump;
			break;

		default:
			String	strUnknownDump = getHexString(abData);
			strMessage = "unknown Meta event: " + strUnknownDump;
			break;

		}
		return strMessage;
	}
	 */



}



/*** DumpReceiver.java ***/



