package receiver;

/*
 *	MidiInDump.java
 *
 *	This file is part of jsresources.org
 */

/*
 * Copyright (c) 1999 - 2001 by Matthias Pfisterer
 * Copyright (c) 2003 by Florian Bomers
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
|<---            this code is formatted to fit into 80 columns             --->|
 */


import java.util.*;
import javax.sound.midi.Transmitter;
import javax.sound.midi.Receiver;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiSystem;




/** 
 * Enables the sending of events created by the MIDI input device to be received by 
 * the designated receiver.
 * @author Jason. Park
 * @author Ki-Hyun. Sung
 * @author Edmund. Zurita
 * */

public class MidiInDump
{

	private static boolean		DEBUG = true;

	/**
	 * 
	 * @param args - user specified argument
	 * @return A vector containing all the MIDI devices connected to the computer.
	 * @throws Exception
	 */
	public static Vector<String> showDevices(String[] args)
	throws Exception
	{

		/*
		 *	The device name/index to listen to.
		 */
/*		String	strDeviceName = null;
		int	nDeviceIndex = -1;*/
		//boolean bUseDefaultSynthesizer = false;
		// TODO: synchronize options with MidiPlayer


		/*
		 *	Parsing of command-line options takes place...
		 */

		/*		Getopt	g = new Getopt("MidiInDump", args, "hlsd:n:D");
		int	c;
		while ((c = g.getopt()) != -1)
			{
				switch (c)
					{
					case 'h':
						printUsageAndExit();

					case 'l':
						MidiCommon.listDevicesAndExit(true, false);

						System.out.println();

					case 'd':
						strDeviceName = g.getOptarg();
						if (DEBUG) { out("MidiInDump.main(): device name: " + strDeviceName); }
						out(strDeviceName);
						break;

					case 'n':

						DeviceIndex = Integer.parseInt(g.getOptarg());
						selectDevice(nDeviceIndex);
						if (DEBUG) { out("MidiInDump.main(): device index: " + nDeviceIndex); }
						break;

					case 'D':
						DEBUG = true;
						break;

					case '?':
						printUsageAndExit();

					default:
						out("MidiInDump.main(): getopt() returned " + c);
						break;
					}
			}*/

		return MidiCommon.listDevicesAndExit(true, false);

	}
	/**
	 * Selects the MIDI device with the index number nDeviceIndex
	 * @param nDeviceIndex - the index of the device to be selected.
	 * @return A Receiver for the device. 
	 */
	public static Receiver selectDevice(int nDeviceIndex){

		if ( nDeviceIndex < 0)
		{
			out("device name/index not specified!");
			//printUsageAndExit();
		}

		MidiDevice.Info	info;
		info = MidiCommon.getMidiDeviceInfo(nDeviceIndex);

		MidiDevice	inputDevice = null;

		try
		{
			inputDevice = MidiSystem.getMidiDevice(info);
			//inputDevice.close();
			//out(inputDevice.toString());
/*			if (  !(inputDevice instanceof Sequencer) && ! (inputDevice instanceof Synthesizer)) {
				// we're now sure that device represents a MIDI port
				// ...
				out("yes");
			}*/
			inputDevice.open();
		}
		catch (MidiUnavailableException e)
		{
			out(e);
		}
		if (inputDevice == null)
		{
			out("wasn't able to retrieve MidiDevice");
			//System.exit(1);
		}
		Receiver	r = new DumpReceiver(System.out, false);

		try
		{
			Transmitter	t = inputDevice.getTransmitter();
			//System.out.println(t.toString());
			t.setReceiver(r);

			//System.out.println(t.getReceiver().toString());
		}
		catch (MidiUnavailableException e)
		{
			out("wasn't able to connect the device's Transmitter to the Receiver:");
			out(e); 
			inputDevice.close();

			//System.exit(1);
		}
		return r;
	}


/*	public static void run(MidiDevice inputDevice, DumpReceiver r){

		//out("now running; interupt the program with [ENTER] when finished");

		//boolean done=false;
		//boolean gettingSequence=true;
		//int counter=0;
		//while(!gettingSequence){
			
			//getSequence(r);
			//if(counter%2==0){
			//out("play a sequence of chords and notes then press enter, or press q to quit");


							try{
					System.out.println("This is Sys.in.read : "+System.in.read());
				}
				catch (IOException ioe){
				}


		//}
			
		if(noteOffEvent(r)){
			System.out.println(getSequence(r));
		}
		
		//}

		if(done){

			inputDevice.close();

		}


		//out("Received "+((DumpReceiver) r).seCount+" sysex messages with a total of "+((DumpReceiver) r).seByteCount+" bytes");
		//out("Received "+((DumpReceiver) r).smCount+" short messages with a total of "+((DumpReceiver) r).smByteCount+" bytes");
		//out("Received a total of "+(((DumpReceiver) r).smByteCount + ((DumpReceiver) r).seByteCount)+" bytes");

					try
				{
					Thread.sleep(1000);
				}
			catch (InterruptedException e)
				{
					if (DEBUG) { out(e); }
			}	


	}*/
	
/*	private static boolean noteOffEvent(DumpReceiver r){
		
		for(int i=0; i<r.m.size(); i++){
			ShortMessage msg = (ShortMessage)r.m.get(i);
			if(msg.getCommand()==128){
				return true;
			}
		}
		
		return false;
	}*/

	/**
	 * Gets the sequence of notes received by the Receiver and returns a string representation
	 * after running chord and/or note recognition.
	 * Clears the sequence contained in the receiver to prepare for the input of the next sequence.
	 * @param r - The Receiver which keeps the ShortMessages of the sequence.
	 * @return The string representation of the sequence.
	 */
	public static String getSequence(Receiver r){
		String returnStr = MidiCommon.recognizeChord(((DumpReceiver)r).m);
		//System.out.println(returnStr);
		((DumpReceiver)r).m.clear();
		((DumpReceiver)r).n.clear();

		return returnStr;
	}

	/*	public static void recognizeChord(Receiver r){

		(DumpReceiver)r.seCount

	}*/

/*	private static String readInput(){

		try
		{
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));


			return in.readLine(); 


		}
		catch(Exception er) { er.printStackTrace(); }
		return null;


	}*/



/*	private static void printUsageAndExit()
	{
		out("MidiInDump: usage:");
		out("  java MidiInDump -h");
		out("    gives help information");
		out("  java MidiInDump -l");
		out("    lists available MIDI devices");
		out("  java MidiInDump [-D] [-d <input device name>] [-n <device index>]");
		out("    -d <input device name>\treads from named device (see '-l')");
		out("    -n <input device index>\treads from device with given index(see '-l')");
		out("    -D\tenables debugging output");
		//System.exit(1);
	}*/

	/**
	 * Utility method which emulates System.out.println()
	 * @param strMessage - the string to be printed.
	 */

	private static void out(String strMessage)
	{
		System.out.println(strMessage);
	}


	/**
	 * Utility method which emulates System.out.println() in debug mode.
	 * @param t
	 */
	private static void out(Throwable t)
	{
		if (DEBUG) {
			t.printStackTrace();
		} else {
			out(t.toString());
		}
	}
}




/*** MidiInDump.java ***/


